@@ -2,7 +2,7 @@
 * ts.cpp: TS packet manipulation
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: ts.cpp,v 1.2 2002/03/17 23:20:22 bozo Exp $
+* $Id: ts.cpp,v 1.2.4.2 2003/11/16 17:49:08 nitrox Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *
@@ -174,6 +174,49 @@ s64 C_TsPacket::GetPCRTime() const
 
 
 
+//------------------------------------------------------------------------------//
+//------------------------------------------------------------------------------// GetLength of Payload.
+//------------------------------------------------------------------------------
+u8 C_TsPacket::GetPayloadLength() const
+{
+  if (!HasAdaptationField())
+    return (TS_PACKET_LEN - 4); //184
+  else
+    return (TS_PACKET_LEN - 5 - GetAdaptationFieldLength());
+}
+
+//------------------------------------------------------------------------------//
+//------------------------------------------------------------------------------// Returns a pointer to the start of the payload
+//------------------------------------------------------------------------------
+byte *C_TsPacket::GetPayloadStart()
+{
+  if (!HasAdaptationField())
+    return (&bData[4]);
+  else
+  {
+    uint8_t pos = 5 + GetAdaptationFieldLength();
+    ASSERT( pos <= TS_PACKET_LEN );
+    return (&bData[pos]);
+  }
+}
+
+//------------------------------------------------------------------------------//
+//------------------------------------------------------------------------------// Adaptation field detection.
+//------------------------------------------------------------------------------
+bool C_TsPacket::HasAdaptationField() const
+{
+    uint8_t iByte = bData[3];
+    return (iByte & 0x20);
+}
+
+uint8_t C_TsPacket::GetAdaptationFieldLength() const
+{
+    if (HasAdaptationField())
+        return (bData[4]);
+    else
+        return 0;
+}
+
 //------------------------------------------------------------------------------
 // BuildHeader: Build a TS header and return its size
 //------------------------------------------------------------------------------
@@ -219,9 +262,13 @@ u8 C_TsPacket::BuildAdaptionField(u64 iPCR)
   bData[4] = 1+6;
   // PCR_base is the only extension present in the adaption field
   bData[5] = 0x10;
+
   // PCR_base value
   SET_U32_TO(bData[6], (iPCR >> 1) & 0xFFFFFFFF);
-  bData[10] = (iPCR & 0x1);
+
+  /* last bit of PCR, and PCR_ext to 0 */
+  bData[10] = (iPCR & 0x01) << 7;
+  bData[11] = 0;
 
   return 4+2+6;
 }
