@@ -2,7 +2,7 @@
 * tsmux.cpp: TS multiplexer
 *-------------------------------------------------------------------------------
 * (c)1999-2002 VideoLAN
-* $Id: tsmux.cpp,v 1.5.4.1 2003/02/18 12:29:10 nitrox Exp $
+* $Id: tsmux.cpp,v 1.5.4.3 2003/11/17 23:53:55 nitrox Exp $
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
@@ -75,7 +75,7 @@ C_TsMuxPmtDecoder::C_TsMuxPmtDecoder(C_NetList *pTsProvider,
                         C_DvbPsiPmtDecoder(iProgramNumber, pTsProvider, this)
 {
   ASSERT(pMux);
-
+  uiPnr = iProgramNumber;
   m_pMux = pMux;
 }
 
@@ -359,8 +359,43 @@ void C_TsMux::HandlePacket(C_TsPacket* pPacket)
 
     if(pDecoder)
     {
+
+      // We got a small problem here:
+      // 13818-1 standard doesn't specify that a PMT PID refers to only 1
+      // program. So we need to check the program number too.
+        uint16_t uiPnr = ((uint8_t)*(pPacket->GetPayloadStart()+4) << 8) |
+                        (uint8_t)*(pPacket->GetPayloadStart()+5);
+      // In all the cases we need to pass the packet to the libdvbpsi
+      // if we don't, it'll complain because of the discontinuity
+      // libdvbpsi just ignores this packet
       pPacket->Ref();
       pDecoder->HandlePacket(pPacket);
+      m_pTsProvider->ReleasePacket(pPacket);
+
+      // In case it was not the right program number, we don't send the packet
+      // Decoders usually don't like receiving PMT refering to a non existant
+      // program
+      if( uiPnr != pDecoder->GetPnr()) return;
+
+      // Here we are, this PMT refers to the right streams. But we can't send
+      // it: we need to regenerate the continuity counter... Let's generate
+      // all the PMT packets ...
+
+      if(pDecoder->GetCurrentPmt())
+      {
+          pDecoder->GetCurrentPmt()->Generate();
+          pDecoder->GetCurrentPmt()->SetPid(iPid);
+          pDecoder->GetCurrentPmt()->TsReset();
+          while(pDecoder->GetCurrentPmt()->TsHasNext())
+          {
+            C_TsPacket *pTsPmt = m_pTsProvider->GetPacket();
+            ASSERT(pTsPmt);
+            pDecoder->GetCurrentPmt()->TsWrite(pTsPmt);
+            m_pHandler->HandlePacket(pTsPmt);
+          }
+      }
+      // if the PMT has not been decoded yet ... just skip it
+      return;
     }
 
     m_pHandler->HandlePacket(pPacket);
